import Acquisition
import AccessControl
import OFS
from ImageFile import ImageFile
from Globals import HTMLFile, MessageDialog, Persistent
import re
import string

import AI_Ext

class ActiveImage(OFS.Folder.Folder):

  """ ActiveImage """

  name='ActiveImage'
  id='ActiveImage'
  icon='misc_/ActiveImages/ActiveImage'
  meta_type='ActiveImage'

  _properties = (
    {'id':'title','type':'string','mode':'w'},
    {'id':'off','type':'string','mode':'w'},
    {'id':'on','type':'string','mode':'w'},
    {'id':'rollover','type':'boolean','mode':'w'},
    {'id':'sticky','type':'boolean','mode':'w'},
    {'id':'group','type':'string','mode':'w'},
    {'id':'ActivePaths','type':'lines','mode':'w'},
    {'id':'triggers','type':'lines','mode':'w'},
    {'id':'link','type':'string','mode':'w'},
    {'id':'target','type':'string','mode':'w'},
    {'id':'NewWindow','type':'boolean','mode':'w'},
    {'id':'features','type':'string','mode':'w'},
    )

  manage_options = (
    #{'label':'My Label','icon':icon,'action':'manage_main','target':'manage_main'},
    {'label': 'Properties',		'action': 'manage_properties',		'target':'_self'},
    {'label': 'Triggers',		'action': 'manage_triggers',		'target':'_self'},
    #{'label': 'Properties(Actual)',	'action': 'manage_propertiesForm',	'target':'_self'},
    {'label': 'Linkage',		'action': 'manage_link',		'target':'_self'},
    {'label': 'Security',		'action': 'manage_access',		'target':'_self'},
    {'label': 'Undo',			'action': 'manage_UndoForm',		'target':'_self'},
    {'label': 'Ownership',		'action': 'manage_owner',		'target':'_self'},
  ) 

  __ac_permissions__=( # what permissions make sense for us? 
    ('View management screens', ('manage_properties','index_html','manage_tabs','manage_main',
                                 'manage_triggers','manage_chooser','manage_link',
                                )),
    ('Change permissions',     ('manage_access')  ),
    ('Change ActiveImages',    ('manage_linkSave','manage_propSave','manage_trigSave')  ),
    ('View ActiveImages',      ('') , ('Anonymous','Manager') )
    )

  def __init__(self,id,title,off='',on='',rollover=1,sticky='',group='',ActivePaths=[],triggers=[],link='',target='',NewWindow='',features=''):
    self.id = id
    self.title = title
    self.off = off
    self.on = on
    self.rollover = rollover
    self.sticky = sticky
    self.group = group
    self.ActivePaths = ActivePaths
    self.triggers = triggers
    self.link = link
    self.target = target
    self.NewWindow = NewWindow
    self.features = features

  def __str__(self):
    """ render tag: <dtml-var "myActiveImages.myActiveImage"> """
    return self.tag()

  """
  ' All methods bebelow are in addition to the standard methods with the
  ' exception of index_html since it must be called after manage_properties
  """
  
  manage_properties=HTMLFile('manage/properties',globals())
  manage_triggers=HTMLFile('manage/triggers',globals())
  manage_chooser=HTMLFile('manage/chooser',globals())
  manage_link=HTMLFile('manage/link',globals())

  index_html=manage_properties

  jsSafe=AI_Ext.jsSafe
  jsClean=AI_Ext.jsClean

  def tag(self,title='^',off='^',on='^',rollover='^',sticky='^',group='^',ActivePaths='^',triggers='^',link='^',target='^',NewWindow='^',features='^'):
    """ display the <img> tag """

    # vars
    onMouseOver=[]
    onMouseOut=[]
    onMouseDown=''
    AI_sticky=''
    AI_newWin=''
    name=''
    REQUEST=self.REQUEST
    comma=re.compile(r",$")
    jsid=self.jsSafe( self.absolute_url() )
    if not REQUEST.has_key('AI_args'):
      REQUEST.set('AI_args',{})
    if not REQUEST['AI_args'].has_key(jsid):
      REQUEST['AI_args'][jsid]={}

    # Default to previously given arg or self value
    if title=='^':
      if REQUEST['AI_args'][jsid].has_key('title'):
        title=REQUEST['AI_args'][jsid]['title']
      else:
        title=self.title_or_id()
    if off=='^':
      if REQUEST['AI_args'][jsid].has_key('off'):
        off=REQUEST['AI_args'][jsid]['off']
      else:
        off=self.off
    if on=='^':
      if REQUEST['AI_args'][jsid].has_key('on'):
        on=REQUEST['AI_args'][jsid]['on']
      else:
        on=self.on
    if rollover=='^':
      if REQUEST['AI_args'][jsid].has_key('rollover'):
        rollover=REQUEST['AI_args'][jsid]['rollover']
        if rollover==0:
          rollover=''
      else:
        rollover=self.rollover
    if sticky=='^':
      if REQUEST['AI_args'][jsid].has_key('sticky'):
        sticky=REQUEST['AI_args'][jsid]['sticky']
        if sticky==0:
          sticky=''
      else:
        sticky=self.sticky
    if group=='^':
      if REQUEST['AI_args'][jsid].has_key('group'):
        group=REQUEST['AI_args'][jsid]['group']
      else:
        group=self.group
    if ActivePaths=='^':
      # Note: ActivePaths gets turned into self.ActivePaths by ActivePath()
      if REQUEST['AI_args'][jsid].has_key('ActivePaths'):
        ActivePaths=REQUEST['AI_args'][jsid]['ActivePaths']
    if triggers=='^':
      # Note: triggers gets turned into self.triggers by getTriggers()
      if REQUEST['AI_args'][jsid].has_key('triggers'):
        triggers=REQUEST['AI_args'][jsid]['triggers']
    if link=='^':
      if REQUEST['AI_args'][jsid].has_key('link'):
        link=REQUEST['AI_args'][jsid]['link']
      else:
        link=self.link
    if target=='^':
      if REQUEST['AI_args'][jsid].has_key('target'):
        target=REQUEST['AI_args'][jsid]['target']
      else:
        target=self.target
    if NewWindow=='^':
      if REQUEST['AI_args'][jsid].has_key('NewWindow'):
        NewWindow=REQUEST['AI_args'][jsid]['NewWindow']
        if NewWindow==0:
          NewWindow=''
      else:
        NewWindow=self.NewWindow
    if features=='^':
      if REQUEST['AI_args'][jsid].has_key('features'):
        features=REQUEST['AI_args'][jsid]['features']
      else:
        features=self.features

    # default target
    if target=='':
      target='_self'

    # build attributes (src, rollover, sticky)
    ActivePath=self.ActivePath(ActivePaths)
    if ActivePath:
      src=on
    else:
      src=off
      if rollover:
        onMouseOver.append( "'%s','%s'" % ( jsid, self.jsSafe(on)  ) )
        onMouseOut.append ( "'%s','%s'" % ( jsid, self.jsSafe(off) ) )
    
    # build attributes for Triggers
    for trigger in self.getTriggers(triggers):
 
      # resolve the trigger
      obj=REQUEST.resolve_url(trigger[0])
      obj_jsid=self.jsSafe( obj.absolute_url() )

      # check if the item is an ActivePath (use overriding args if given)
      if REQUEST['AI_args'].has_key(obj_jsid) and REQUEST['AI_args'][obj_jsid].has_key('ActivePaths'):
        obj_ActivePaths=REQUEST['AI_args'][obj_jsid]['ActivePaths']
      else:
        obj_ActivePaths=list(obj.ActivePaths)

      # don't trigger it if it's an ActivePath
      if not obj.ActivePath(obj_ActivePaths):
        onMouseOver.append( "'%s','%s'" % ( obj_jsid, self.jsSafe(trigger[1]) ) )
        onMouseOut.append ( "'%s','%s'" % ( obj_jsid, self.jsSafe(obj.off)    ) )

    # Rollovers: build onMouseOver and onMouseOut strings
    if len(onMouseOver)>0:
      onMouseOver=string.join(onMouseOver,',')
      onMouseOver='onMouseOver="AI_changeImages(%s)"' % (onMouseOver)
    else:
      onMouseOver=''
    if len(onMouseOut)>0:
      onMouseOut=string.join(onMouseOut,',')
      onMouseOut='onMouseOut="AI_changeImages(%s)"' % (onMouseOut)
    else:
      onMouseOut=''

    # Sticky Images: build onMouseDown string
    if sticky and not ActivePath and not rollover:
      AI_sticky="'%s'" % (jsid)
      if group:
        # prefix group with the ActiveImage absolute_url so as to avoid group name clashing
        AI_sticky="%s,'%s'" % ( AI_sticky, self.jsSafe(self.aq_parent.absolute_url()+'_'+group) )
      AI_sticky="AI_sticky(%s);" % (AI_sticky)
    if link:
      if NewWindow:
        AI_newWin="AI_newWin('%s','%s','%s');" % (link,target,features)
    if AI_sticky or AI_newWin:
      onMouseDown='onMouseDown="%s%s"' % (AI_sticky,AI_newWin)

    # build name ("if not ActivePath" keeps a sticky group trigger from happenning)
    if not ActivePath:
      name='name="%s"' % jsid

    # build <img> tag
    tag='<img src="%s" %s alt="%s" border=0>' % (src, name, self.title_or_id())
     
    # if we don't need the <a href> tag (if no linking and no JavaScript)
    if not link and not onMouseOver and not onMouseOut and not onMouseDown:
      pass
    else:
      # determine attributes
      if link:
        if NewWindow:
          # AI_newWin will take care of it
          link='#'
          target=''
        else:
          # use target attribute
          target='target="%s"' % (target)
      else:
        # used for sticky
        link='#'
        target=''
      # build  <a href> tag
      tag='<a href="%s" %s %s %s %s >%s</a>' % (link,target,onMouseOver,onMouseOut,onMouseDown,tag)
     
    return tag

  def js(self,title='^',off='^',on='^',rollover='^',sticky='^',group='^',ActivePaths='^',triggers='^',link='^',target='^',NewWindow='^',features='^'):
    """ preload images """

    # vars
    js='\n'
    jsid=self.jsSafe(self.absolute_url())
    ImageURLs=[]
    REQUEST=self.REQUEST
    if not REQUEST.has_key('AI_PreLoaded'):
      REQUEST.set('AI_PreLoaded',[])
    if not REQUEST.has_key('AI_args'):
      REQUEST.set('AI_args',{})
    if not REQUEST['AI_args'].has_key(jsid):
      REQUEST['AI_args'][jsid]={}
      
    # Default to self value if variable wan't passed. It new value passed, make note.
    if title=='^':
      title=self.title_or_id()
    else:
      REQUEST['AI_args'][jsid]['title']=title
    if off=='^':
      off=self.off
    else:
      REQUEST['AI_args'][jsid]['off']=off
    if on=='^':
      on=self.on
    else:
      REQUEST['AI_args'][jsid]['on']=on
    if rollover=='^':
      rollover=self.rollover
    else:
      if rollover==0:
        rollover=''
      REQUEST['AI_args'][jsid]['rollover']=rollover
    if sticky=='^':
      sticky=self.sticky
    else:
      if sticky==0:
        sticky=''
      REQUEST['AI_args'][jsid]['sticky']=sticky
    if group=='^':
     group=self.group
    else:
      REQUEST['AI_args'][jsid]['group']=group
    if ActivePaths != '^':
      REQUEST['AI_args'][jsid]['ActivePaths']=ActivePaths
      # Note: ActivePaths gets turned into self.ActivePaths by ActivePath()
    if triggers != '^':
      REQUEST['AI_args'][jsid]['triggers']=triggers
      # Note: triggers gets turned into self.triggers by getTriggers()
    if link=='^':
      link=self.link
    else:
      REQUEST['AI_args'][jsid]['link']=link
    if target=='^':
      target=self.target
    else:
      REQUEST['AI_args'][jsid]['target']=target
    if NewWindow=='^':
      NewWindow=self.NewWindow
    else:
      REQUEST['AI_args'][jsid]['NewWindow']=NewWindow
    if features=='^':
      features=self.features
    else:
      REQUEST['AI_args'][jsid]['features']=features

    # get list of all image URLs
    # 1. don't print if the rollover is turned off 
    # 2. don't print if we are on an ActivePath
    # 3. print if sticky

    # rollover
    if rollover and not self.ActivePath(ActivePaths) or sticky:
      ImageURLs.append(on)
      ImageURLs.append(off)

    # triggers
    for trigger in self.getTriggers(triggers):

      # resolve the trigger
      obj=REQUEST.resolve_url(trigger[0])
      obj_jsid=self.jsSafe( obj.absolute_url() )

      # check if the item is an ActivePath (use overriding args if given)
      if REQUEST['AI_args'].has_key(obj_jsid) and REQUEST['AI_args'][obj_jsid].has_key('ActivePaths'):
        obj_ActivePaths=REQUEST['AI_args'][obj_jsid]['ActivePaths']
      else:
        obj_ActivePaths=list(obj.ActivePaths)

      # don't trigger it if it's an ActivePath
      if not obj.ActivePath(obj_ActivePaths):
        ImageURLs.append(obj.off)
        ImageURLs.append(trigger[1])

    # pre-load individual images
    for url in ImageURLs:
      url_jsid=self.jsSafe(url)
      if url_jsid not in REQUEST.AI_PreLoaded:
        js=js + '    ' + '// Image at ' + url + '\n'
        js=js + '    ' + url_jsid + ' = new Image();\n'
        js=js + '    ' + url_jsid + '.src = "' + url + '";\n\n'
        REQUEST.AI_PreLoaded.append(url_jsid)

    # Sticky Stuff (Don't print if not needed)
    if sticky and not self.ActivePath(ActivePaths) and not rollover:

      # ActiveImage JavaScript object
      js=js + '    ' + '// ActiveImage at ' + self.absolute_url() + '\n'
      js=js + '    ' + jsid + '_on = ' + self.jsSafe(on) + ';\n'
      js=js + '    ' + jsid + '_off = ' + self.jsSafe(off) + ';\n\n'
  
      # Sticky Group Entry
      if group:
        js_group=self.jsSafe(self.aq_parent.absolute_url()+'_'+group)
        js=js+"    // Add Sticky Group for %s\n" \
              "    if (! window.s_groups['%s']){ s_groups['%s'] = [] }\n" \
              "    pos = s_groups['%s'].length;\n" \
              "    s_groups['%s'][pos] = '%s';\n" % (self.absolute_url(), js_group, js_group, js_group, js_group, jsid)

    return js

  def ActivePath(self,paths=''):
    """ check if we are on an ActivePath """
    arr=[]
    status=None
    REQUEST=self.REQUEST
    if type(paths) != type(arr):
      ActivePaths=list(self.ActivePaths)
    else:
      ActivePaths=paths
    SearchPath=REQUEST.SCRIPT_NAME+REQUEST.PATH_INFO
    for path in ActivePaths:
      path=string.strip(path)
      if path and string.find(SearchPath, path) > -1: 
        status=1
    return status

  def getTriggers(self,triggers=''):
    """ parse triggers and return list of pairs """
    t=[]
    arr=[]
    count=0
    # if a triggers argument is passed, it gets passed back unchanged
    if type(triggers)==type(arr):
      return triggers
    else:
      triggers=list(self.triggers)
      size=len(triggers)/2
      if (len(triggers) % 2)==1:
       triggers.append('')
      for x in range(0, size):
        t.append([ triggers[0], triggers[1] ])
        triggers.pop(0)
        triggers.pop(0)
      return t

  def displayName(self,name):
    """ clean up a name for display """
    name=string.lower(name)
    name=string.capitalize(name)
    name=string.replace(name,'_',' ')
    return name

  def manage_linkSave(self,REQUEST):
    """ save link, target and features properties """
    features=[]

    # yes/no options
    toggle=['toolbar','location','status','menubar','scrollbars','resizable'] 
    for item in toggle:
      if REQUEST.has_key(item):
        features.append(item+'=yes')
      else:
        features.append(item+'=no')

    # width and height
    if REQUEST.width:
      features.append('width='+REQUEST.width)
    if REQUEST.height:
      features.append('height='+REQUEST.height)

    # create features string
    features=string.join(features,',')

    # save and return
    self.manage_changeProperties({ 'link': REQUEST.link, 'target': REQUEST.target, 'NewWindow': REQUEST.NewWindow, 'features': features })

    return MessageDialog(
      title  ='Success!',
      message='Your changes have been saved',
      action='manage_link'
    )

  def manage_propSave(self,REQUEST):
    """ save properties """
    self.manage_changeProperties(REQUEST)
    if not REQUEST.has_key('chooser_redirect'):
      REQUEST.set('chooser_redirect',REQUEST.HTTP_REFERER)
    return MessageDialog(
      title  ='Success!',
      message='Your changes have been saved',
      action=REQUEST.chooser_redirect
    )

  def manage_chooser_CHECKED(self,ci,au):
    """ return CHECKED if on the current item """
    # trigger
    t=re.compile(r"^trigger_")
    if t.match(ci):
      triggers=list(self.triggers)
      num=string.split(ci,'_')
      num=int(num[1])-1
      if triggers[num] == au:
        return 'CHECKED'
      else:
        return ''
    # property
    else:
      if self.getProperty(ci) == au:
        return 'CHECKED'
      else:
        return ''

  def manage_trigSave(self,REQUEST,RESPONSE):
    """ update triggers """
 
    # vars
    t=re.compile(r"^trigger_")
    triggers=list(self.triggers)

    # add new trigger
    if REQUEST.has_key('add'):
      triggers.append(' ')
      triggers.append(' ')
      self.triggers=triggers
      RESPONSE.redirect('manage_triggers')
  
    # delete some items
    elif REQUEST.has_key('delete'):

      # error checking
      if not REQUEST.get('deleteList'):
        return MessageDialog(
          title  ='No items were specified!',
          message="No items were specified!<br><br>",
          action ='manage_triggers')

      # mark 'on' items for deletion
      dList1=REQUEST.get('deleteList')
             
      # mark 'off' items for deletion
      dList2=[]
      for num in dList1:
        dList2.append(int(num)-1)
      
      # join the list (convert all to int)
      dList=dList1+dList2
      count=0
      for num in dList:
        dList[count]=int(num)
        count=count+1
      #return dList
      
      # make new triggers
      count=0
      triggersNew=[]
      for trigger in triggers:
        if count not in dList:
          triggersNew.append(trigger)
        count=count+1
          
      # save and return
      self.triggers=triggersNew
      return MessageDialog(
        title  ='Success!',
        message='Your changes have been saved',
        action ='manage_triggers')

    # manage_chooser update
    elif REQUEST.has_key('chooser'):
      for n,v in REQUEST.form.items():
        if t.match(n):
          name=n
          value=v
      num=string.split(name,'_')
      num=int(num[1])-1
      triggers[num]=value
      self.triggers=triggers
      return MessageDialog(
        title  ='Success!',
        message='Your changes have been saved',
        action ='manage_triggers')

    # manage_triggers update
    elif REQUEST.has_key('save'):

      # vars
      keys=[]
      triggers=[]
 
      # get list of keys
      for key in REQUEST.form.keys():
        if t.search(key):
          keys.append(key)

      # sort keys. Note: start at 1 not 0 since "trigger_1" is the first fieldname
      for num in range(1, (len(keys)+1) ):
        triggers.append( REQUEST.form.get('trigger_'+str(num)) )
   
      self.triggers=triggers

      return MessageDialog(
        title  ='Success!',
        message='Your changes have been saved',
        action ='manage_triggers')

    
addActiveImageForm=HTMLFile('manage/addActiveImageForm',globals())

def addActiveImage(self,id,title,off='',on='',rollover=1,sticky='',group='',ActivePaths=[],triggers=[],link='',target='',NewWindow='',features='',REQUEST=None):
  """Create a ActiveImage and install it in its parent Folder
  The argument 'self' will be bound to the parent Folder.
  """
  handle=ActiveImage(id,title,off,on,rollover,sticky,group,ActivePaths,triggers,link,target,NewWindow,features)
  handle.id=id
  handle.title=title

  self._setObject(id,handle)
  if REQUEST is not None:
    return self.manage_main(self, REQUEST)
